@@ -3,6 +3,7 @@ from __future__ import annotations
 import logging
 from collections import defaultdict
 
+from django.conf import settings
 from django.contrib.admin import site
 from django.contrib.admin.options import InlineModelAdmin
 from django.contrib.admin.utils import flatten_fieldsets
@@ -10,7 +11,6 @@ from django.contrib.contenttypes.admin import GenericInlineModelAdmin
 from django.db import models
 from django.db.models.fields.reverse_related import ForeignObjectRel
 from django.forms.models import _get_foreign_key
-from django.urls import reverse
 
 from .orm_fields import (
     _AGGREGATES,
@@ -24,40 +24,38 @@ from .orm_fields import (
     OrmFkField,
     OrmFunctionField,
     OrmModel,
+    get_model_name,
 )
 from .query import (
     ASC,
     DSC,
     TYPES,
-    BooleanFieldType,
-    DateFieldType,
-    DateTimeFieldType,
-    NumberFieldType,
-    StringFieldType,
+    BooleanType,
+    BoundQuery,
+    DateTimeType,
+    DateType,
+    NumberType,
+    StringType,
 )
 
 _FIELD_MAP = {
-    models.BooleanField: BooleanFieldType,
-    models.NullBooleanField: BooleanFieldType,
-    models.CharField: StringFieldType,
-    models.TextField: StringFieldType,
-    models.GenericIPAddressField: StringFieldType,
-    models.UUIDField: StringFieldType,
-    models.DateTimeField: DateTimeFieldType,
-    models.DateField: DateFieldType,
-    models.DecimalField: NumberFieldType,
-    models.FloatField: NumberFieldType,
-    models.IntegerField: NumberFieldType,
-    models.AutoField: NumberFieldType,
+    models.BooleanField: BooleanType,
+    models.NullBooleanField: BooleanType,
+    models.CharField: StringType,
+    models.TextField: StringType,
+    models.GenericIPAddressField: StringType,
+    models.UUIDField: StringType,
+    models.DateTimeField: DateTimeType,
+    models.DateField: DateType,
+    models.DecimalField: NumberType,
+    models.FloatField: NumberType,
+    models.IntegerField: NumberType,
+    models.AutoField: NumberType,
 }
 
 
-def get_model_name(model, sep="."):
-    return f"{model._meta.app_label}{sep}{model.__name__}"
-
-
 def _get_all_admin_fields(request):
-    request.data_browser = True
+    request.data_browser = {"calculated_fields": set()}
 
     def from_fieldsets(admin, all_):
         obj = admin.model()  # we want the admin change field sets, not the add ones
@@ -143,10 +141,11 @@ def _get_fields_for_model(model, model_admins, admin_fields):
                     pretty_name=field_name,
                     type_=field_type,
                 )
-            else:
-                logging.getLogger(__name__).warning(
-                    f"{model.__name__}.{field_name} unsupported type {type(field).__name__}"
-                )
+            else:  # pragma: no cover
+                if settings.DEBUG:
+                    logging.getLogger(__name__).warning(
+                        f"{model.__name__}.{field_name} unsupported type {type(field).__name__}"
+                    )
 
     return OrmModel(fields=fields, admin=model_admins[model])
 
@@ -174,7 +173,7 @@ def get_models(request):
 
 
 def _get_django_lookup(field_type, lookup):
-    if field_type == StringFieldType and lookup == "equals":
+    if field_type == StringType and lookup == "equals":
         return "iexact"
     else:
         lookup = {
@@ -208,13 +207,28 @@ def _filter(qs, filter_, filter_str):
         return qs.filter(**{filter_str: filter_.parsed})
 
 
-def get_results(request, bound_query):
-    request.data_browser = True
+def cols_sub_query(bound_query):
+    return BoundQuery(
+        bound_query.model_name,
+        [f for f in bound_query.fields if f.pivoted],
+        [f for f in bound_query.valid_filters if f.orm_bound_field.filter_],
+    )
 
-    if not bound_query.fields:
-        return []
 
-    admin = bound_query.orm_models[bound_query.model_name].admin
+def rows_sub_query(bound_query):
+    return BoundQuery(
+        bound_query.model_name,
+        [
+            f
+            for f in bound_query.fields
+            if f.orm_bound_field.can_pivot and not f.pivoted
+        ],
+        [f for f in bound_query.valid_filters if f.orm_bound_field.filter_],
+    )
+
+
+def _get_results(request, bound_query, orm_models):
+    admin = orm_models[bound_query.model_name].admin
     qs = admin.get_queryset(request)
 
     # functions
@@ -261,10 +275,30 @@ def get_results(request, bound_query):
             sort_fields.append(f"-{field.orm_bound_field.queryset_path}")
     qs = qs.order_by(*sort_fields)
 
+    return qs
+
+
+def get_results(request, bound_query, orm_models):
+    request.data_browser = {"calculated_fields": set()}
+
+    if not bound_query.fields:
+        return {"rows": [], "cols": [], "body": []}
+
+    if bound_query.col_fields and bound_query.row_fields:
+        qs = _get_results(request, bound_query, orm_models)
+        rows_qs = _get_results(request, rows_sub_query(bound_query), orm_models)
+        cols_qs = _get_results(request, cols_sub_query(bound_query), orm_models)
+    else:
+        qs = list(_get_results(request, bound_query, orm_models))
+        rows_qs = qs
+        cols_qs = qs
+
     # gather up all the objects to fetch for calculated fields
     to_load = defaultdict(set)
+    loading_for = defaultdict(set)
     for field in bound_query.bound_fields:
         if field.model_name:
+            loading_for[field.model_name].add(field.name)
             pks = to_load[field.model_name]
             for row in qs:
                 pks.add(row[field.queryset_path])
@@ -272,49 +306,52 @@ def get_results(request, bound_query):
     # fetch all the calculated field objects
     cache = {}
     for model_name, pks in to_load.items():
-        admin = bound_query.orm_models[model_name].admin
+        admin = orm_models[model_name].admin
+        request.data_browser["calculated_fields"] = loading_for[model_name]
         cache[model_name] = admin.get_queryset(request).in_bulk(pks)
 
     # dump out the results
-    def get_admin_link(obj):
-        model_name = get_model_name(obj.__class__, "_")
-        url_name = f"admin:{model_name}_change".lower()
-        url = reverse(url_name, args=[obj.pk])
-        return f'<a href="{url}">{obj}</a>'
+    def format_table(fields, data):
+        results = []
+        for row in data:
+            res_row = {}
+            for field in fields:
+                value = row[field.path_str]
+                if field.model_name:
+                    admin = orm_models[field.model_name].admin
+                    obj = cache[field.model_name].get(value)
+                    value = obj, admin
+                res_row[field.path_str] = field.format(value)
+            results.append(res_row)
+        return results
 
-    results = []
+    def get_fields(row, fields):
+        return tuple((field.path_str, row[field.queryset_path]) for field in fields)
+
+    col_keys = {}
+    for row in cols_qs:
+        col_keys[get_fields(row, bound_query.col_fields)] = None
+
+    row_keys = {}
+    for row in rows_qs:
+        row_keys[get_fields(row, bound_query.row_fields)] = None
+
+    data = defaultdict(dict)
     for row in qs:
-        res_row = []
-        for field in bound_query.bound_fields:
-            value = row[field.queryset_path]
-            if value is None:
-                # null
-                res = None
-            elif not field.model_name:
-                # normal field
-                res = value
-            else:
-                obj = cache[field.model_name][value]
-                if field.admin_link:
-                    # link to admin
-                    res = get_admin_link(obj)
-                else:
-                    tail = field.full_path[-1]
-                    admin = bound_query.orm_models[field.model_name].admin
-                    if hasattr(admin, tail):
-                        # admin callable
-                        func = getattr(admin, tail)
-                        try:
-                            res = func(obj)
-                        except Exception as e:
-                            res = str(e)
-                    else:
-                        # model property or callable
-                        try:
-                            value = getattr(obj, tail)
-                            res = value() if callable(value) else value
-                        except Exception as e:
-                            res = str(e)
-            res_row.append(field.type_.format(res))
-        results.append(res_row)
-    return results
+        row_key = get_fields(row, bound_query.row_fields)
+        col_key = get_fields(row, bound_query.col_fields)
+        data[row_key][col_key] = dict(get_fields(row, bound_query.data_fields))
+
+    body = []
+    blank = {field.path_str: None for field in bound_query.data_fields}
+    for col_key in col_keys:
+        table = []
+        for row_key in row_keys:
+            table.append(data[row_key].get(col_key, blank))
+        body.append(format_table(bound_query.data_fields, table))
+
+    return {
+        "rows": format_table(bound_query.row_fields, [dict(row) for row in row_keys]),
+        "cols": format_table(bound_query.col_fields, [dict(col) for col in col_keys]),
+        "body": body,
+    }
