@@ -14,6 +14,7 @@ ASC, DSC = "asc", "dsc"
 @dataclass
 class QueryField:
     path: str
+    pivoted: bool = False
     direction: str = None
     priority: int = None
 
@@ -52,14 +53,22 @@ class Query:
         for field in field_str.split(","):
             field = field.strip()
             if field:
+                # pivot
+                if field.startswith("&"):
+                    field = field[1:]
+                    pivoted = True
+                else:
+                    pivoted = False
+
+                # sort
                 if "+" in field:
                     path, direction, priority = parse_sort(field, "+", ASC)
-                    fields.append(QueryField(path, direction, priority))
+                    fields.append(QueryField(path, pivoted, direction, priority))
                 elif "-" in field:
                     path, direction, priority = parse_sort(field, "-", DSC)
-                    fields.append(QueryField(path, direction, priority))
+                    fields.append(QueryField(path, pivoted, direction, priority))
                 else:
-                    fields.append(QueryField(field, None, None))
+                    fields.append(QueryField(field, pivoted, None, None))
 
         filters = []
         for path__lookup, values in dict(get_args).items():
@@ -74,9 +83,10 @@ class Query:
     def field_str(self):
         field_strs = []
         for field in self.fields:
+            pivot = "&" if field.pivoted else ""
             direction = {ASC: "+", DSC: "-", None: ""}[field.direction]
             priority = str(field.priority) if field.direction else ""
-            field_strs.append(f"{'__'.join(field.path)}{direction}{priority}")
+            field_strs.append(f"{pivot}{'__'.join(field.path)}{direction}{priority}")
         return ",".join(field_strs)
 
     @property
@@ -101,7 +111,7 @@ class Query:
         return f"{base_url}?{params}"
 
 
-class MetaFieldType(type):
+class TypeMeta(type):
     def __repr__(cls):
         return cls.__name__
 
@@ -112,11 +122,11 @@ class MetaFieldType(type):
     @property
     def name(cls):
         name = cls.__name__.lower()
-        assert name.endswith("fieldtype")
-        return name[: -len("fieldtype")]
+        assert name.endswith("type")
+        return name[: -len("type")]
 
 
-class BaseFieldType(metaclass=MetaFieldType):
+class BaseType(metaclass=TypeMeta):
     default_value = None
     lookups = {}
 
@@ -128,11 +138,23 @@ class BaseFieldType(metaclass=MetaFieldType):
         return value
 
     @staticmethod
-    def parse(value):
+    def _parse(value):
         return value
 
+    @classmethod
+    def parse(cls, lookup, value):
+        if lookup not in cls.lookups:
+            return None, f"Bad lookup '{lookup}' expected {cls.lookups}"
+        else:
+            type_ = TYPES[cls.lookups[lookup]]
+            try:
+                return type_._parse(value), None
+            except Exception as e:
+                err_message = str(e) if str(e) else repr(e)
+                return None, err_message
 
-class StringFieldType(BaseFieldType):
+
+class StringType(BaseType):
     default_value = ""
     lookups = {
         "equals": "string",
@@ -149,7 +171,7 @@ class StringFieldType(BaseFieldType):
     }
 
 
-class NumberFieldType(BaseFieldType):
+class NumberType(BaseType):
     default_value = 0
     lookups = {
         "equals": "number",
@@ -166,11 +188,11 @@ class NumberFieldType(BaseFieldType):
         return float(value) if value is not None else None
 
     @staticmethod
-    def parse(value):
+    def _parse(value):
         return float(value)
 
 
-class DateTimeFieldType(BaseFieldType):
+class DateTimeType(BaseType):
     default_value = "now"
     lookups = {
         "equals": "datetime",
@@ -183,7 +205,7 @@ class DateTimeFieldType(BaseFieldType):
     }
 
     @staticmethod
-    def parse(value):
+    def _parse(value):
         if value.lower().strip() == "now":
             return timezone.now()
         return timezone.make_aware(dateutil.parser.parse(value))
@@ -193,7 +215,7 @@ class DateTimeFieldType(BaseFieldType):
         return str(timezone.make_naive(value)) if value else None
 
 
-class DateFieldType(BaseFieldType):
+class DateType(BaseType):
     default_value = "today"
     lookups = {
         "equals": "date",
@@ -206,7 +228,7 @@ class DateFieldType(BaseFieldType):
     }
 
     @staticmethod
-    def parse(value):
+    def _parse(value):
         if value.lower().strip() == "today":
             return timezone.now().date()
         return timezone.make_aware(dateutil.parser.parse(value)).date()
@@ -216,53 +238,73 @@ class DateFieldType(BaseFieldType):
         return str(value) if value else None
 
 
-class WeekDayFieldType(BaseFieldType):
-    @staticmethod
-    def format(value):
-        days = [
-            None,
-            "Sunday",
-            "Monday",
-            "Tuesday",
-            "Wednesday",
-            "Thursday",
-            "Friday",
-            "Saturday",
-        ]
-        return days[value] if value else None
+class WeekDayType(BaseType):
+    default_value = "Monday"
+    lookups = {"equals": "weekday", "not_equals": "weekday"}
+
+    _days = [
+        "Sunday",
+        "Monday",
+        "Tuesday",
+        "Wednesday",
+        "Thursday",
+        "Friday",
+        "Saturday",
+    ]
+
+    @classmethod
+    def format(cls, value):
+        return cls._days[value - 1] if value else None
+
+    @classmethod
+    def _parse(cls, value):
+        for i, v in enumerate(cls._days):
+            if v.lower()[:3] == value.lower()[:3]:
+                return i + 1
+        raise Exception("not a day of the week")
 
 
-class MonthFieldType(BaseFieldType):
-    @staticmethod
-    def format(value):
-        days = [
-            None,
-            "January",
-            "Feburary",
-            "March",
-            "April",
-            "May",
-            "June",
-            "July",
-            "August",
-            "September",
-            "October",
-            "November",
-            "December",
-        ]
-        return days[value] if value else None
+class MonthType(BaseType):
+    default_value = "January"
+    lookups = {"equals": "month", "not_equals": "month"}
+
+    _months = [
+        "January",
+        "Feburary",
+        "March",
+        "April",
+        "May",
+        "June",
+        "July",
+        "August",
+        "September",
+        "October",
+        "November",
+        "December",
+    ]
+
+    @classmethod
+    def format(cls, value):
+        return cls._months[value - 1] if value else None
+
+    @classmethod
+    def _parse(cls, value):
+        for i, v in enumerate(cls._months):
+            if v.lower()[:3] == value.lower()[:3]:
+                return i + 1
+        raise Exception("not a month")
 
 
-class HTMLFieldType(BaseFieldType):
+class HTMLType(BaseType):
     pass
 
 
-class BooleanFieldType(BaseFieldType):
+class BooleanType(BaseType):
     default_value = True
     lookups = {"equals": "boolean", "not_equals": "boolean", "is_null": "boolean"}
 
     @staticmethod
-    def parse(value):
+    def _parse(value):
         value = value.lower()
         if value == "true":
             return True
@@ -272,7 +314,7 @@ class BooleanFieldType(BaseFieldType):
             raise ValueError("Expected 'true' or 'false'")
 
 
-TYPES = {field_type.name: field_type for field_type in BaseFieldType.__subclasses__()}
+TYPES = {field_type.name: field_type for field_type in BaseType.__subclasses__()}
 
 
 class BoundFieldMixin:
@@ -284,6 +326,10 @@ class BoundFieldMixin:
     def pretty_path(self):
         return self.orm_bound_field.pretty_path
 
+    @property
+    def path_str(self):
+        return self.orm_bound_field.path_str
+
 
 @dataclass
 class BoundFilter(BoundFieldMixin):
@@ -296,38 +342,37 @@ class BoundFilter(BoundFieldMixin):
         return cls(orm_bound_field, query_filter.lookup, query_filter.value)
 
     def __post_init__(self):
-        self.parsed = None
-        self.err_message = None
-
-        lookups = self.orm_bound_field.type_.lookups
-        if self.lookup not in lookups:
-            self.err_message = f"Bad lookup '{self.lookup}' expected {lookups}"
-        else:
-            type_ = TYPES[lookups[self.lookup]]
-            try:
-                self.parsed = type_.parse(self.value)
-            except Exception as e:
-                self.err_message = str(e) if str(e) else repr(e)
-
+        self.parsed, self.err_message = self.orm_bound_field.type_.parse(
+            self.lookup, self.value
+        )
         self.is_valid = not self.err_message
 
 
 @dataclass
 class BoundField(BoundFieldMixin):
     orm_bound_field: Any
+    pivoted: bool
     direction: Optional[str]
     priority: Optional[int]
 
     @classmethod
     def bind(cls, orm_bound_field, query_field):
-        concrete = orm_bound_field.concrete
-        direction = query_field.direction if concrete else None
-        priority = query_field.priority if concrete else None
-        return cls(orm_bound_field, direction, priority)
+        return cls(
+            orm_bound_field,
+            query_field.pivoted and orm_bound_field.can_pivot,
+            query_field.direction if orm_bound_field.concrete else None,
+            query_field.priority if orm_bound_field.concrete else None,
+        )
 
 
 class BoundQuery:
-    def __init__(self, query, orm_models):
+    def __init__(self, model_name, fields, filters):
+        self.model_name = model_name
+        self.fields = fields
+        self.filters = filters
+
+    @classmethod
+    def bind(cls, query, orm_models):
         def get_orm_field(parts):
             model_name = query.model_name
             orm_bound_field = None
@@ -341,20 +386,21 @@ class BoundQuery:
                 return None
             return orm_bound_field
 
-        self.model_name = query.model_name
-        self.orm_models = orm_models
+        model_name = query.model_name
 
-        self.fields = []
+        fields = []
         for query_field in query.fields:
             orm_bound_field = get_orm_field(query_field.path)
             if orm_bound_field:
-                self.fields.append(BoundField.bind(orm_bound_field, query_field))
+                fields.append(BoundField.bind(orm_bound_field, query_field))
 
-        self.filters = []
+        filters = []
         for query_filter in query.filters:
             orm_bound_field = get_orm_field(query_filter.path)
             if orm_bound_field and orm_bound_field.concrete:
-                self.filters.append(BoundFilter.bind(orm_bound_field, query_filter))
+                filters.append(BoundFilter.bind(orm_bound_field, query_filter))
+
+        return cls(model_name, fields, filters)
 
     @property
     def sort_fields(self):
@@ -371,3 +417,29 @@ class BoundQuery:
     @property
     def bound_filters(self):
         return [f.orm_bound_field for f in self.valid_filters]
+
+    @property
+    def col_fields(self):
+        return [field.orm_bound_field for field in self.fields if field.pivoted]
+
+    @property
+    def row_fields(self):
+        if self.col_fields:
+            return [
+                field.orm_bound_field
+                for field in self.fields
+                if field.orm_bound_field.can_pivot and not field.pivoted
+            ]
+        else:
+            return [field.orm_bound_field for field in self.fields]
+
+    @property
+    def data_fields(self):
+        if self.col_fields:
+            return [
+                field.orm_bound_field
+                for field in self.fields
+                if not field.orm_bound_field.can_pivot
+            ]
+        else:
+            return []
