@@ -5,28 +5,33 @@ from data_browser import orm, orm_fields
 from data_browser.query import (
     ASC,
     DSC,
-    BooleanFieldType,
-    BoundFilter,
+    BooleanType,
     BoundQuery,
-    DateFieldType,
-    DateTimeFieldType,
-    MonthFieldType,
-    NumberFieldType,
+    DateTimeType,
+    DateType,
+    MonthType,
+    NumberType,
     Query,
     QueryField,
     QueryFilter,
-    StringFieldType,
-    WeekDayFieldType,
+    StringType,
+    WeekDayType,
 )
 from django.http import QueryDict
 from django.utils import timezone
 
+from .util import ANY
+
 
 @pytest.fixture
 def query():
     return Query(
         "app.model",
-        [QueryField("fa", ASC, 1), QueryField("fd", DSC, 0), QueryField("fn")],
+        [
+            QueryField("fa", False, ASC, 1),
+            QueryField("fd", False, DSC, 0),
+            QueryField("fn"),
+        ],
         [QueryFilter("bob", "equals", "fred")],
     )
 
@@ -34,21 +39,21 @@ def query():
 @pytest.fixture
 def orm_models():
     return {
-        "string": orm._get_fields_for_type(StringFieldType),
-        "number": orm._get_fields_for_type(NumberFieldType),
+        "string": orm._get_fields_for_type(StringType),
+        "number": orm._get_fields_for_type(NumberType),
         "app.model": orm_fields.OrmModel(
             fields={
                 "fa": orm_fields.OrmConcreteField(
                     model_name="app.model",
                     name="fa",
                     pretty_name="fa",
-                    type_=StringFieldType,
+                    type_=StringType,
                 ),
                 "fd": orm_fields.OrmConcreteField(
                     model_name="app.model",
                     name="fd",
                     pretty_name="fd",
-                    type_=StringFieldType,
+                    type_=StringType,
                 ),
                 "fn": orm_fields.OrmCalculatedField(
                     model_name="app.model", name="fn", pretty_name="fn"
@@ -57,13 +62,13 @@ def orm_models():
                     model_name="app.model",
                     name="bob",
                     pretty_name="bob",
-                    type_=StringFieldType,
+                    type_=StringType,
                 ),
                 "num": orm_fields.OrmConcreteField(
                     model_name="app.model",
                     name="num",
                     pretty_name="num",
-                    type_=NumberFieldType,
+                    type_=NumberType,
                 ),
                 "tom": orm_fields.OrmFkField(
                     model_name="app.model",
@@ -80,7 +85,7 @@ def orm_models():
                     model_name="app.Tom",
                     name="jones",
                     pretty_name="jones",
-                    type_=StringFieldType,
+                    type_=StringType,
                 ),
                 "michael": orm_fields.OrmFkField(
                     model_name="app.Tom",
@@ -97,7 +102,7 @@ def orm_models():
                     model_name="app.Michael",
                     name="bolton",
                     pretty_name="bolton",
-                    type_=StringFieldType,
+                    type_=StringType,
                 )
             },
             admin=True,
@@ -107,7 +112,7 @@ def orm_models():
 
 @pytest.fixture
 def bound_query(query, orm_models):
-    return BoundQuery(query, orm_models)
+    return BoundQuery.bind(query, orm_models)
 
 
 class TestQuery:
@@ -145,15 +150,19 @@ class TestQuery:
 
     def test_from_request_field_no_name(self):
         q = Query.from_request("app.model", "+2", QueryDict(""))
-        assert q == Query("app.model", [QueryField("", ASC, 2)], [])
+        assert q == Query("app.model", [QueryField("", False, ASC, 2)], [])
 
     def test_from_request_field_no_priority(self):
         q = Query.from_request("app.model", "fn+", QueryDict(""))
-        assert q == Query("app.model", [QueryField("fn", None, None)], [])
+        assert q == Query("app.model", [QueryField("fn")], [])
 
     def test_from_request_field_bad_priority(self):
         q = Query.from_request("app.model", "fn+x", QueryDict(""))
-        assert q == Query("app.model", [QueryField("fn", None, None)], [])
+        assert q == Query("app.model", [QueryField("fn")], [])
+
+    def test_from_request_field_pivoted(self):
+        q = Query.from_request("app.model", "&fn", QueryDict(""))
+        assert q == Query("app.model", [QueryField("fn", True)], [])
 
     def test_url(self, query):
         assert (
@@ -180,37 +189,47 @@ class TestBoundQuery:
 
     def test_bad_field(self, orm_models):
         query = Query("app.model", [QueryField("yata")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == []
 
     def test_bad_fk(self, orm_models):
         query = Query("app.model", [QueryField("yata__yata")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == []
 
     def test_bad_fk_field(self, orm_models):
         query = Query("app.model", [QueryField("tom__yata")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == []
 
     def test_bad_fk_field_aggregate(self, orm_models):
         query = Query("app.model", [QueryField("tom__jones__yata")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == []
 
     def test_bad_long_fk(self, orm_models):
         query = Query("app.model", [QueryField("yata__yata__yata")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == []
 
     def test_aggregate(self, orm_models):
         query = Query("app.model", [QueryField("tom__jones__count")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == [["tom", "jones", "count"]]
 
+    def test_piovt_aggregate(self, orm_models):
+        query = Query("app.model", [QueryField("tom__jones__count", pivoted=True)], [])
+        bound_query = BoundQuery.bind(query, orm_models)
+        assert [f.pivoted for f in bound_query.fields] == [False]
+
+    def test_piovt(self, orm_models):
+        query = Query("app.model", [QueryField("tom__jones", pivoted=True)], [])
+        bound_query = BoundQuery.bind(query, orm_models)
+        assert [f.pivoted for f in bound_query.fields] == [True]
+
     def test_bad_filter(self, orm_models):
         query = Query("app.model", [], [QueryFilter("yata", "equals", "fred")])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.filters] == []
 
     def test_bad_filter_value(self, orm_models):
@@ -219,119 +238,126 @@ class TestBoundQuery:
             [],
             [QueryFilter("num", "equals", "fred"), QueryFilter("num", "equals", 1)],
         )
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.value for f in bound_query.filters] == ["fred", 1]
         assert [f.value for f in bound_query.valid_filters] == [1]
 
     def test_fk(self, orm_models):
         query = Query("app.model", [QueryField("tom")], [])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.fields] == []
 
     def test_filter_calculated_field(self, orm_models):
         query = Query("app.model", [], [QueryFilter("fn", "equals", "fred")])
-        bound_query = BoundQuery(query, orm_models)
+        bound_query = BoundQuery.bind(query, orm_models)
         assert [f.path for f in bound_query.filters] == []
 
 
-class TestFieldType:
+class TestType:
     def test_repr(self):
-        assert repr(StringFieldType) == f"StringFieldType"
+        assert repr(StringType) == f"StringType"
 
 
-class TestStringFieldType:
+class TestStringType:
     def test_validate(self):
-        orm_bound_field = orm_fields.OrmBoundField([], [], StringFieldType)
-        assert BoundFilter(orm_bound_field, "contains", "hello").is_valid
-        assert not BoundFilter(orm_bound_field, "pontains", "hello").is_valid
+        assert StringType.parse("contains", "hello") == ("hello", None)
+        assert StringType.parse("pontains", "hello") == (None, ANY(str))
 
     def test_default_lookup(self):
-        assert StringFieldType.default_lookup == "equals"
+        assert StringType.default_lookup == "equals"
 
     def test_format(self):
-        assert StringFieldType.format("bob") == "bob"
+        assert StringType.format("bob") == "bob"
 
 
-class TestNumberFieldType:
+class TestNumberType:
     def test_validate(self):
-        orm_bound_field = orm_fields.OrmBoundField([], [], NumberFieldType)
-        assert BoundFilter(orm_bound_field, "gt", "6.1").is_valid
-        assert not BoundFilter(orm_bound_field, "pontains", "6.1").is_valid
-        assert not BoundFilter(orm_bound_field, "gt", "hello").is_valid
-        assert BoundFilter(orm_bound_field, "is_null", "True").is_valid
-        assert not BoundFilter(orm_bound_field, "is_null", "hello").is_valid
+        assert NumberType.parse("gt", "6.1") == (6.1, None)
+        assert NumberType.parse("pontains", "6.1") == (None, ANY(str))
+        assert NumberType.parse("gt", "hello") == (None, ANY(str))
+        assert NumberType.parse("is_null", "True") == (True, None)
+        assert NumberType.parse("is_null", "hello") == (None, ANY(str))
 
     def test_default_lookup(self):
-        assert NumberFieldType.default_lookup == "equals"
+        assert NumberType.default_lookup == "equals"
 
     def test_format(self):
-        assert NumberFieldType.format(6) == 6
+        assert NumberType.format(6) == 6
 
 
-class TestDateTimeFieldType:
+class TestDateTimeType:
     def test_validate(self):
-        orm_bound_field = orm_fields.OrmBoundField([], [], DateTimeFieldType)
-        assert BoundFilter(orm_bound_field, "gt", "2018-03-20T22:31:23").is_valid
-        assert not BoundFilter(orm_bound_field, "gt", "hello").is_valid
-        assert not BoundFilter(
-            orm_bound_field, "pontains", "2018-03-20T22:31:23"
-        ).is_valid
-        assert BoundFilter(orm_bound_field, "is_null", "True").is_valid
-        assert not BoundFilter(orm_bound_field, "is_null", "hello").is_valid
-        assert BoundFilter(orm_bound_field, "gt", "now").is_valid
+        assert DateTimeType.parse("gt", "2018-03-20T22:31:23") == (ANY(datetime), None)
+        assert DateTimeType.parse("gt", "hello") == (None, ANY(str))
+        assert DateTimeType.parse("pontains", "2018-03-20T22:31:23") == (None, ANY(str))
+        assert DateTimeType.parse("is_null", "True") == (True, None)
+        assert DateTimeType.parse("is_null", "hello") == (None, ANY(str))
+        assert DateTimeType.parse("gt", "now") == (ANY(datetime), None)
 
     def test_default_lookup(self):
-        assert DateTimeFieldType.default_lookup == "equals"
+        assert DateTimeType.default_lookup == "equals"
 
     def test_format(self):
         assert (
-            DateTimeFieldType.format(
-                timezone.make_aware(datetime(2020, 5, 19, 8, 42, 16))
-            )
+            DateTimeType.format(timezone.make_aware(datetime(2020, 5, 19, 8, 42, 16)))
             == "2020-05-19 08:42:16"
         )
 
 
-class TestDateFieldType:
+class TestDateType:
     def test_validate(self):
-        orm_bound_field = orm_fields.OrmBoundField([], [], DateFieldType)
-        assert BoundFilter(orm_bound_field, "gt", "2018-03-20T22:31:23").is_valid
-        assert not BoundFilter(orm_bound_field, "gt", "hello").is_valid
-        assert not BoundFilter(
-            orm_bound_field, "pontains", "2018-03-20T22:31:23"
-        ).is_valid
-        assert BoundFilter(orm_bound_field, "is_null", "True").is_valid
-        assert not BoundFilter(orm_bound_field, "is_null", "hello").is_valid
-        assert BoundFilter(orm_bound_field, "gt", "today").is_valid
+        assert DateType.parse("gt", "2018-03-20T22:31:23") == (ANY(date), None)
+        assert DateType.parse("gt", "hello") == (None, ANY(str))
+        assert DateType.parse("pontains", "2018-03-20T22:31:23") == (None, ANY(str))
+        assert DateType.parse("is_null", "True") == (True, None)
+        assert DateType.parse("is_null", "hello") == (None, ANY(str))
+        assert DateType.parse("gt", "today") == (ANY(date), None)
 
     def test_default_lookup(self):
-        assert DateFieldType.default_lookup == "equals"
+        assert DateType.default_lookup == "equals"
 
     def test_format(self):
-        assert DateFieldType.format(date(2020, 5, 19)) == "2020-05-19"
+        assert DateType.format(date(2020, 5, 19)) == "2020-05-19"
 
 
-class TestWeekDayFieldType:
-    def test_format(self):
-        assert WeekDayFieldType.format(None) is None
-        assert WeekDayFieldType.format(1) == "Sunday"
-        assert WeekDayFieldType.format(7) == "Saturday"
-
-
-class TestMonthFieldType:
-    def test_format(self):
-        assert MonthFieldType.format(None) is None
-        assert MonthFieldType.format(1) == "January"
-        assert MonthFieldType.format(12) == "December"
-
-
-class TestBooleanFieldType:
+class TestWeekDayType:
     def test_validate(self):
-        orm_bound_field = orm_fields.OrmBoundField([], [], BooleanFieldType)
-        assert BoundFilter(orm_bound_field, "equals", "True").is_valid
-        assert BoundFilter(orm_bound_field, "equals", "False").is_valid
-        assert not BoundFilter(orm_bound_field, "equals", "hello").is_valid
-        assert not BoundFilter(orm_bound_field, "pontains", "True").is_valid
+        assert WeekDayType.parse("equals", "Sunday") == (1, None)
+        assert WeekDayType.parse("equals", "Saturday") == (7, None)
+        assert WeekDayType.parse("equals", "Bob") == (None, ANY(str))
+        assert WeekDayType.parse("gt", "Monday") == (None, ANY(str))
+
+    def test_format(self):
+        assert WeekDayType.format(None) is None
+        assert WeekDayType.format(1) == "Sunday"
+        assert WeekDayType.format(7) == "Saturday"
 
     def test_default_lookup(self):
-        assert BooleanFieldType.default_lookup == "equals"
+        assert WeekDayType.default_lookup == "equals"
+
+
+class TestMonthType:
+    def test_validate(self):
+        assert MonthType.parse("equals", "January") == (1, None)
+        assert MonthType.parse("equals", "December") == (12, None)
+        assert MonthType.parse("equals", "Bob") == (None, ANY(str))
+        assert MonthType.parse("gt", "January") == (None, ANY(str))
+
+    def test_format(self):
+        assert MonthType.format(None) is None
+        assert MonthType.format(1) == "January"
+        assert MonthType.format(12) == "December"
+
+    def test_default_lookup(self):
+        assert MonthType.default_lookup == "equals"
+
+
+class TestBooleanType:
+    def test_validate(self):
+        assert BooleanType.parse("equals", "True") == (True, None)
+        assert BooleanType.parse("equals", "False") == (False, None)
+        assert BooleanType.parse("equals", "hello") == (None, ANY(str))
+        assert BooleanType.parse("pontains", "True") == (None, ANY(str))
+
+    def test_default_lookup(self):
+        assert BooleanType.default_lookup == "equals"
